@@ -5,11 +5,10 @@ use v5.20;
 use experimental qw/ signatures /;
 package YAML::Tidy;
 
-our $VERSION = '0.001'; # VERSION
+our $VERSION = '0.002'; # VERSION
 
 use YAML::Tidy::Node;
 use YAML::Tidy::Config;
-use YAML::LibYAML::API;
 use YAML::LibYAML::API::XS;
 use YAML::PP::Common qw/
     YAML_PLAIN_SCALAR_STYLE YAML_SINGLE_QUOTED_SCALAR_STYLE
@@ -37,14 +36,14 @@ sub cfg($self) { $self->{cfg} }
 sub tidy($self, $yaml) {
     local $Data::Dumper::Sortkeys = 1;
     my @lines = split /\n/, $yaml, -1;
-    my $tree = $self->tree($yaml, \@lines);
+    my $tree = $self->_tree($yaml, \@lines);
     $self->{lines} = \@lines;
-    $self->process(undef, $tree);
+    $self->_process(undef, $tree);
     $yaml = join "\n", @{ $self->{lines} };
     return $yaml;
 }
 
-sub process($self, $parent, $node) {
+sub _process($self, $parent, $node) {
     my $type = $node->{type} || '';
     if ($node->{flow}) {
         return;
@@ -71,7 +70,7 @@ sub process($self, $parent, $node) {
     if ($pre and $trimtrailing) {
         if (defined $pre->{line} and $pre->{line} <= $start->{line}) {
             my ($from, $to) = ($pre->{line}, $start->{line});
-            $self->trim($from, $to);
+            $self->_trim($from, $to);
         }
         if ($type eq 'DOC') {
         }
@@ -81,7 +80,7 @@ sub process($self, $parent, $node) {
         my $ignore_firstlevel = ($self->{partial} and $level == 0);
         if ($level < 0 or $ignore_firstlevel) {
             for my $c (@{ $node->{children} }) {
-                $self->process($node, $c);
+                $self->_process($node, $c);
             }
             return;
         }
@@ -116,11 +115,11 @@ sub process($self, $parent, $node) {
         my $diff = $indent - $realindent;
 #        warn __PACKAGE__.':'.__LINE__.$".Data::Dumper->Dump([\$diff], ['diff']);
         if ($diff) {
-            $self->fix_indent($node, $diff, $col);
+            $self->_fix_indent($node, $diff, $col);
             $node->fix_node_indent($diff);
         }
         for my $c (@{ $node->{children} }) {
-            $self->process($node, $c);
+            $self->_process($node, $c);
         }
         return;
     }
@@ -132,51 +131,68 @@ sub process($self, $parent, $node) {
         if ($node->empty_scalar) {
             return;
         }
-        my $endline = $node->{end}->{line} - 1;
-        my $nextline = $node->{nextline};
-        my $new_indent = $parent->indent + $indent;
-
-        my $new_spaces = ' ' x $new_indent;
         if ($node->{name} eq 'alias_event') {
             return;
         }
+        my $new_indent = $parent->indent + $indent;
+        my $new_spaces = ' ' x $new_indent;
+
+        my ($anchor, $tag, $comments, $scalar) = $self->_find_scalar_start($node);
+#        warn __PACKAGE__.':'.__LINE__.$".Data::Dumper->Dump([\$lines], ['lines']);
+#        warn __PACKAGE__.':'.__LINE__.$".Data::Dumper->Dump([\$scalar], ['scalar']);
+        my $explicit_indent = 0;
+        if ($scalar->[2] =~ m/[>|]/) {
+            my $l = $lines->[ $scalar->[0] ];
+            my ($ind) = substr($l, $scalar->[1]) =~ m/^[|>][+-]?([0-9]*)/;
+            $explicit_indent = $ind;
+        }
+        my $skipfirst = 0;
+        my $before = substr($line, 0, $col);
+        if ($before =~ tr/ \t//c) {
+            # same line as key
+            $before =~ s/[\t ]+$/ /;
+            $line = $before . substr($line, $col);
+            $lines->[ $startline ] = $line;
+            $skipfirst = 1;
+        }
+        my $realstart = $scalar->[0];
+        if ($trimtrailing) {
+            $self->_trim($startline, $realstart);
+        }
+        for my $i ($startline .. $realstart) {
+            my $line = $lines->[ $i ];
+            if ($i == $startline and $col > 0) {
+                my $before = substr($line, 0, $col);
+                if ($before =~ tr/ //c) {
+                    next;
+                }
+            }
+            unless ($line =~ tr/ //c) {
+                next;
+            }
+            $line =~ s/^ */$new_spaces/;
+            $lines->[ $i] = $line;
+        }
+        # leave alone explicitly indented block scalars
+        return if $explicit_indent;
+
+        $startline = $realstart;
+        my $endline = $node->realendline;
+
+        my $line = $lines->[ $startline ];
+        my $realcol = $scalar->[1];
+        $col = $realcol;
+
+        my $nextline = $node->{nextline};
+
         my $block = ($node->{style} eq YAML_LITERAL_SCALAR_STYLE
             or $node->{style} eq YAML_FOLDED_SCALAR_STYLE);
-        my $explicit_indent = 0;
-        if ($node->{style} != YAML_PLAIN_SCALAR_STYLE) {
-            while ($startline < $endline) {
-                if ($trimtrailing) {
-                    $self->trim($startline, $startline);
-                }
-                if ($node->{style} != YAML_PLAIN_SCALAR_STYLE and $lines->[ $startline ] =~ m/(?:^|\t| )([>|"'])/g) {
-                    my $pos = pos $lines->[ $startline ];
-                    $col = $pos - 1;
-                    last;
-                }
-                $startline++;
-            }
-        }
-        my $line = $lines->[ $startline ];
-        my $before = substr($line, 0, $col);
         if ($block) {
-            my ($ind) = substr($line, $col) =~ m/^[|>][+-]?([0-9]*)/;
-            # leave alone explicitly indented block scalars
-            return if $ind;
 
-            if ($before =~ tr/ \t//c) {
-                # same line as key
-                $before =~ s/[\t ]+$/ /;
-                $line = $before . substr($line, $col);
-                $lines->[ $startline ] = $line;
-                $startline++;
-            }
-            else {
-                $lines->[ $startline ] =~ s/^ +/$new_spaces/;
-                $startline++;
-            }
+            $startline++;
             while ($startline < $endline and $lines->[ $startline ] !~ tr/ //c) {
                 if ($trimtrailing) {
-                    $self->trim($startline, $startline);
+                    $self->_trim($startline, $startline);
                 }
                 $startline++;
             }
@@ -229,14 +245,8 @@ sub process($self, $parent, $node) {
         elsif ($node->{style} == YAML_PLAIN_SCALAR_STYLE or
                 $node->{style} == YAML_SINGLE_QUOTED_SCALAR_STYLE or
                 $node->{style} == YAML_DOUBLE_QUOTED_SCALAR_STYLE) {
+            $startline++ if $skipfirst;
             $endline = $node->{end}->{line};
-            if ($before =~ tr/ //c) {
-                # same line as key
-                if ($trimtrailing) {
-                    $lines->[ $startline ] =~ s/[\t ]+$//;
-                }
-                $startline++;
-            }
             return if $startline >= @$lines;
             my @slice = @$lines[$startline .. $endline ];
             if ($level == 0 and not $indenttoplevelscalar) {
@@ -260,21 +270,86 @@ sub process($self, $parent, $node) {
     }
 }
 
-sub trim($self, $from, $to) {
+sub _find_scalar_start($self, $node) {
+    my $lines = $self->{lines};
+    my $from = $node->line;
+    my $to = $node->realendline;
+    my $col = $node->indent;
+    my $end = $node->end;
+    my $endcol = $end->{column};
+#    warn __PACKAGE__.':'.__LINE__.$".Data::Dumper->Dump([\$endcol], ['endcol']);
+#    warn __PACKAGE__.':'.__LINE__.$".Data::Dumper->Dump([\$col], ['col']);
+    my @slice = @$lines[ $from .. $to ];
+#    warn __PACKAGE__.':'.__LINE__.$".Data::Dumper->Dump([\@slice], ['slice']);
+    my $anchor;
+    my $tag;
+    my @comments;
+    my $start;
+    my $scalar;
+    for my $i (0 .. $#slice) {
+        my $line = $slice[ $i ];
+        my $f = $i == 0 ? $col : 0;
+        my $t = $i == $#slice ? ($endcol || length($line)) : length($line);
+        my $part = substr($line, $f, $t - $f);
+        if ($part =~ m/^ *(\#.*)$/g) {
+            my $comment = $1;
+            my $pos1 = length($line) - length($comment);
+            push @comments, [$i + $from, $pos1, $comment];
+            next;
+        }
+        my $cur;
+        while ($part =~ m/\G\s*([&!])(\S+)/g) {
+            my $type = $1;
+            my $name = $2;
+            $cur = pos $part;
+            my $pos = $cur - 1;
+            my $pos1 = $pos - length $name;
+            my $prop = substr($part, $pos1, 1+ length $name);
+            if ($type eq '&') {
+                $anchor = [$i + $from, $pos1 + $f, $prop];
+            }
+            elsif ($type eq '!') {
+                $tag = [$i + $from, $pos1 + $f, $prop];
+            }
+        }
+        pos($part) = $cur;
+        if ($part =~ m/\G *(\#.*)$/g) {
+            my $comment = $1;
+            $cur = pos $part;
+            my $pos1 = length($line) - length($comment);
+            push @comments, [$i + $from, $pos1, $comment];
+            next;
+        }
+        pos($part) = $cur;
+        if ($part =~ m/\G *(\S)/g) {
+            $scalar = $1;
+            my $pos1 = (pos $part) - 1;
+            $scalar = [$i + $from, $pos1 + $f, $scalar];
+            last;
+        }
+    }
+    $scalar ||= [$to, length($slice[ -1 ]), ''];
+#    warn __PACKAGE__.':'.__LINE__.$".Data::Dumper->Dump([\$anchor], ['anchor']);
+#    warn __PACKAGE__.':'.__LINE__.$".Data::Dumper->Dump([\$tag], ['tag']);
+#    warn __PACKAGE__.':'.__LINE__.$".Data::Dumper->Dump([\@comments], ['comments']);
+#    warn __PACKAGE__.':'.__LINE__.$".Data::Dumper->Dump([\$scalar], ['scalar']);
+    return ($anchor, $tag, \@comments, $scalar);
+}
+
+sub _trim($self, $from, $to) {
     my $lines = $self->{lines};
     for my $line (@$lines[ $from .. $to ]) {
         $line =~ s/[\t ]+$//;
     }
 }
 
-sub fix_indent($self, $node, $fix, $offset) {
+sub _fix_indent($self, $node, $fix, $offset) {
 #    warn __PACKAGE__.':'.__LINE__.$".Data::Dumper->Dump([\$fix], ['fix']);
 #    warn __PACKAGE__.':'.__LINE__.$".Data::Dumper->Dump([\$offset], ['offset']);
     $offset ||= 0;
     my $startline = $node->line;
     my $lines = $self->{lines};
-    my $endline = $node->{endline};
-    $endline--; # if $endline >= @$lines;
+    my $endline = $node->realendline;
     my @slice = @$lines[$startline .. $endline];
     for my $line (@slice) {
         next unless length $line;
@@ -308,8 +383,8 @@ sub fix_indent($self, $node, $fix, $offset) {
     @$lines[$startline .. $endline] = @slice;
 }
 
-sub tree($self, $yaml, $lines) {
-    my $events = $self->parse($yaml);
+sub _tree($self, $yaml, $lines) {
+    my $events = $self->_parse($yaml);
     $self->{events} = $events;
     my $first = shift @$events;
     my $end = pop @$events;
@@ -379,7 +454,6 @@ sub tree($self, $yaml, $lines) {
         elsif ($name =~ m/_end_event/) {
             my $last = pop @stack;
 
-            $ref->{endline} = $event->{end}->{line};
             $ref->{end} = $event;
 
             $ref = $last;
@@ -401,19 +475,19 @@ sub tree($self, $yaml, $lines) {
             my $nextline = $next->{start}->{line};
             $event->{nextline} = $nextline;
         }
-        pp($event) if DEBUG;
+        _pp($event) if DEBUG;
     }
     $self->{tree} = $docs;
     return $docs;
 }
 
-sub parse($self, $yaml) {
+sub _parse($self, $yaml) {
     my @events;
-    YAML::LibYAML::API::parse_string_events($yaml, \@events);
+    YAML::LibYAML::API::XS::parse_string_events($yaml, \@events);
     return \@events;
 }
 
-sub pp($event) {
+sub _pp($event) {
     my $name = $event->{name};
     my $level = $event->{level};
     $name =~ s/_event$//;
@@ -476,10 +550,14 @@ YAML::Tidy - Clean YAML files
 
 =head1 SYNOPSIS
 
-    % yamltidy in.yaml
+    % cat in.yaml
     a:
         b:
          c: d
+    % yamltidy in.yaml
+    a:
+      b:
+        c: d
 
 For documentation see L<https://github.com/perlpunk/yamltidy>
 
@@ -489,6 +567,35 @@ yamltidy can automatically fix indentation in your YAML files.
 
 For more information, see L<https://github.com/perlpunk/yamltidy>.
 
+=head1 METHODS
+
+=over
+
+=item C<new>
+
+    my $yt = YAML::Tidy->new;
+
+=item C<tidy>
+
+    my $outyaml = $yt->tidy($inyaml);
+
+=item C<highlight>
+
+    my $ansicolored = $yt->highlight($yaml, 'ansi');
+    my $html = $yt->highlight($yaml, 'html');
+
+=item C<cfg>
+
+    my $cfg = $yt->cfg;
+
+Return L<YAML::Tidy::Config>
+
+=back
+
+=head1 AUTHOR
+
+Tina Müller E<lt>tinita@cpan.orgE<gt>
+
 =head1 SEE ALSO
 
 =over
@@ -497,7 +604,11 @@ For more information, see L<https://github.com/perlpunk/yamltidy>.
 
 =item perltidy L<Perl::Tidy>
 
-=item L<YAML::PP::LibYAML>
+=item L<YAML::LibYAML::API>
+
+=item L<https://github.com/yaml/libyaml>
+
+=item L<https://www.yaml.info/>
 
 =back
 
